@@ -168,7 +168,7 @@ if ($formdata = $mform2->is_cancelled()) {
     $allowdeletes      = (!empty($formdata->uuallowdeletes) and $optype != UU_USER_ADDNEW and $optype != UU_USER_ADDINC);
     $allowsuspends     = (!empty($formdata->uuallowsuspends));
     $bulk              = $formdata->uubulk;
-    $noemailduplicates = $formdata->uunoemailduplicates;
+    $noemailduplicates = empty($CFG->allowaccountssameemail) ? 1 : $formdata->uunoemailduplicates;
     $standardusernames = $formdata->uustandardusernames;
     $resetpasswords    = isset($formdata->uuforcepasswordchange) ? $formdata->uuforcepasswordchange : UU_PWRESET_NONE;
 
@@ -187,7 +187,8 @@ if ($formdata = $mform2->is_cancelled()) {
     // caches
     $ccache         = array(); // course cache - do not fetch all courses here, we  will not probably use them all anyway!
     $cohorts        = array();
-    $rolecache      = uu_allowed_roles_cache(); // roles lookup cache
+    $rolecache      = uu_allowed_roles_cache(); // Course roles lookup cache.
+    $sysrolecache   = uu_allowed_sysroles_cache(); // System roles lookup cache.
     $manualcache    = array(); // cache of used manual enrol plugins in each course
     $supportedauths = uu_supported_auths(); // officially supported plugins that are enabled
 
@@ -231,8 +232,8 @@ if ($formdata = $mform2->is_cancelled()) {
                 if (isset($USER->$key) and is_array($USER->$key)) {
                     // this must be some hacky field that is abusing arrays to store content and format
                     $user->$key = array();
-                    $user->$key['text']   = $value;
-                    $user->$key['format'] = FORMAT_MOODLE;
+                    $user->{$key['text']}   = $value;
+                    $user->{$key['format']} = FORMAT_MOODLE;
                 } else {
                     $user->$key = trim($value);
                 }
@@ -566,8 +567,18 @@ if ($formdata = $mform2->is_cancelled()) {
                     }
                     if ($existinguser->$column !== $user->$column) {
                         if ($column === 'email') {
-                            if ($DB->record_exists('user', array('email'=>$user->email))) {
-                                if ($noemailduplicates) {
+                            $select = $DB->sql_like('email', ':email', false, true, false, '|');
+                            $params = array('email' => $DB->sql_like_escape($user->email, '|'));
+                            if ($DB->record_exists_select('user', $select , $params)) {
+
+                                $changeincase = core_text::strtolower($existinguser->$column) === core_text::strtolower(
+                                                $user->$column);
+
+                                if ($changeincase) {
+                                    // If only case is different then switch to lower case and carry on.
+                                    $user->$column = core_text::strtolower($user->$column);
+                                    continue;
+                                } else if ($noemailduplicates) {
                                     $upt->track('email', $stremailduplicate, 'error');
                                     $upt->track('status', $strusernotupdated, 'error');
                                     $userserrors++;
@@ -850,6 +861,15 @@ if ($formdata = $mform2->is_cancelled()) {
                         $cohort = $DB->get_record('cohort', array('id'=>$addcohort));
                     } else {
                         $cohort = $DB->get_record('cohort', array('idnumber'=>$addcohort));
+                        if (empty($cohort) && has_capability('moodle/cohort:manage', context_system::instance())) {
+                            // Cohort was not found. Create a new one.
+                            $cohortid = cohort_add_cohort((object)array(
+                                'idnumber' => $addcohort,
+                                'name' => $addcohort,
+                                'contextid' => context_system::instance()->id
+                            ));
+                            $cohort = $DB->get_record('cohort', array('id'=>$cohortid));
+                        }
                     }
 
                     if (empty($cohort)) {
@@ -880,6 +900,41 @@ if ($formdata = $mform2->is_cancelled()) {
         // find course enrolments, groups, roles/types and enrol periods
         // this is again a special case, we always do this for any updated or created users
         foreach ($filecolumns as $column) {
+            if (preg_match('/^sysrole\d+$/', $column)) {
+
+                if (!empty($user->$column)) {
+                    $sysrolename = $user->$column;
+                    if ($sysrolename[0] == '-') {
+                        $removing = true;
+                        $sysrolename = substr($sysrolename, 1);
+                    } else {
+                        $removing = false;
+                    }
+
+                    if (array_key_exists($sysrolename, $sysrolecache)) {
+                        $sysroleid = $sysrolecache[$sysrolename]->id;
+                    } else {
+                        $upt->track('enrolments', get_string('unknownrole', 'error', s($sysrolename)), 'error');
+                        continue;
+                    }
+
+                    if ($removing) {
+                        if (user_has_role_assignment($user->id, $sysroleid, SYSCONTEXTID)) {
+                            role_unassign($sysroleid, $user->id, SYSCONTEXTID);
+                            $upt->track('enrolments', get_string('unassignedsysrole',
+                                    'tool_uploaduser', $sysrolecache[$sysroleid]->name));
+                        }
+                    } else {
+                        if (!user_has_role_assignment($user->id, $sysroleid, SYSCONTEXTID)) {
+                            role_assign($sysroleid, $user->id, SYSCONTEXTID);
+                            $upt->track('enrolments', get_string('assignedsysrole',
+                                    'tool_uploaduser', $sysrolecache[$sysroleid]->name));
+                        }
+                    }
+                }
+
+                continue;
+            }
             if (!preg_match('/^course\d+$/', $column)) {
                 continue;
             }
@@ -918,32 +973,32 @@ if ($formdata = $mform2->is_cancelled()) {
                 // let's not invent new lang strings here for this rarely used feature.
 
                 if (!empty($user->{'role'.$i})) {
-                    $addrole = $user->{'role'.$i};
-                    if (array_key_exists($addrole, $rolecache)) {
-                        $rid = $rolecache[$addrole]->id;
+                    $rolename = $user->{'role'.$i};
+                    if (array_key_exists($rolename, $rolecache)) {
+                        $roleid = $rolecache[$rolename]->id;
                     } else {
-                        $upt->track('enrolments', get_string('unknownrole', 'error', s($addrole)), 'error');
+                        $upt->track('enrolments', get_string('unknownrole', 'error', s($rolename)), 'error');
                         continue;
                     }
 
-                    role_assign($rid, $user->id, context_course::instance($courseid));
+                    role_assign($roleid, $user->id, context_course::instance($courseid));
 
                     $a = new stdClass();
                     $a->course = $shortname;
-                    $a->role   = $rolecache[$rid]->name;
+                    $a->role   = $rolecache[$roleid]->name;
                     $upt->track('enrolments', get_string('enrolledincourserole', 'enrol_manual', $a));
                 }
 
             } else if ($manual and $manualcache[$courseid]) {
 
                 // find role
-                $rid = false;
+                $roleid = false;
                 if (!empty($user->{'role'.$i})) {
-                    $addrole = $user->{'role'.$i};
-                    if (array_key_exists($addrole, $rolecache)) {
-                        $rid = $rolecache[$addrole]->id;
+                    $rolename = $user->{'role'.$i};
+                    if (array_key_exists($rolename, $rolecache)) {
+                        $roleid = $rolecache[$rolename]->id;
                     } else {
-                        $upt->track('enrolments', get_string('unknownrole', 'error', s($addrole)), 'error');
+                        $upt->track('enrolments', get_string('unknownrole', 'error', s($rolename)), 'error');
                         continue;
                     }
 
@@ -956,14 +1011,14 @@ if ($formdata = $mform2->is_cancelled()) {
                     } else if (empty($formdata->{'uulegacy'.$addtype})) {
                         continue;
                     } else {
-                        $rid = $formdata->{'uulegacy'.$addtype};
+                        $roleid = $formdata->{'uulegacy'.$addtype};
                     }
                 } else {
                     // no role specified, use the default from manual enrol plugin
-                    $rid = $manualcache[$courseid]->roleid;
+                    $roleid = $manualcache[$courseid]->roleid;
                 }
 
-                if ($rid) {
+                if ($roleid) {
                     // Find duration and/or enrol status.
                     $timeend = 0;
                     $status = null;
@@ -990,11 +1045,11 @@ if ($formdata = $mform2->is_cancelled()) {
                         $timeend = $today + $manualcache[$courseid]->enrolperiod;
                     }
 
-                    $manual->enrol_user($manualcache[$courseid], $user->id, $rid, $today, $timeend, $status);
+                    $manual->enrol_user($manualcache[$courseid], $user->id, $roleid, $today, $timeend, $status);
 
                     $a = new stdClass();
                     $a->course = $shortname;
-                    $a->role   = $rolecache[$rid]->name;
+                    $a->role   = $rolecache[$roleid]->name;
                     $upt->track('enrolments', get_string('enrolledincourserole', 'enrol_manual', $a));
                 }
             }
@@ -1132,7 +1187,10 @@ while ($linenum <= $previewrows and $fields = $cir->next()) {
         if (!validate_email($rowcols['email'])) {
             $rowcols['status'][] = get_string('invalidemail');
         }
-        if ($DB->record_exists('user', array('email'=>$rowcols['email']))) {
+
+        $select = $DB->sql_like('email', ':email', false, true, false, '|');
+        $params = array('email' => $DB->sql_like_escape($rowcols['email'], '|'));
+        if ($DB->record_exists_select('user', $select , $params)) {
             $rowcols['status'][] = $stremailduplicate;
         }
     }
